@@ -19,12 +19,12 @@ use Term::ANSIColor;
 use Log::Fast;
 use Time::HiRes qw(time);
 use File::Basename;
-use Best [
- [
-   'Data::Printer' => {'args' => {'colored' => TRUE,'use_prototypes' => FALSE}},
-   'Data::Dumper::Simple',
-   'Data::Dumper'
- ]
+use Best [ # Loads the first available module, in order of preference.
+    [
+        'Data::Printer' => {'args' => {'colored' => TRUE,'use_prototypes' => FALSE}},
+        'Data::Dumper::Simple',
+        'Data::Dumper'
+    ]
 ];
 
 our $Modules = {
@@ -40,7 +40,7 @@ BEGIN {
     require Exporter;
 
     # set the version for version checking
-    our $VERSION = 0.34;
+    our $VERSION = 0.36;
 
     # Inherit from Exporter to export functions and variables
     our @ISA = qw(Exporter);
@@ -52,13 +52,20 @@ BEGIN {
     our @EXPORT_OK = qw(@Levels $Modules);
 } ## end BEGIN
 
-# Not used, but optionally exported if the coder wants to use it as some sort
-# of index or reference.
+# This can be optionally exported for whatever
 our @Levels = qw( ERR WARN NOTICE INFO VERBOSE DEBUG DEBUGMAX DEBUGWAIT );
 
+# For quick level checks to speed up execution
+our %LevelLogic;
+for (my $count=0;$count<scalar(@Levels);$count++) {
+    $LevelLogic{$Levels[$count]} = $count;
+}
+
 my %ANSILevel   = ();                     # Global debug level colorized messages hash.  It will be filled in later.
 my $LOG         = Log::Fast->global();    # Let's snag the Log::Fast object.
 my $MASTERSTART = time;                   # Script start timestamp.
+my ($SCRIPTNAME, $SCRIPTPATH, $suffix) = fileparse($0);
+my $PARENT      = $$;                     # Get the parent process ID
 
 =head1 NAME
 
@@ -83,8 +90,8 @@ Debug::Easy - A Handy Debugging Module With Colorized Output and Formatting
  #  NOTICE    = Notice
  #  INFO      = Information
  #  VERBOSE   = Special version of INFO that does not output any
- #              Logging headings and goes to STDOUT instead of STDERR.
- #              Very useful for verbose modes in your scripts.
+ #              Logging headings.  Very useful for verbose modes in your
+ #              scripts.
  #  DEBUG     = Level 1 Debugging messages
  #  DEBUGMAX  = Level 2 Debugging messages (typically more terse)
  #
@@ -106,7 +113,19 @@ Debug::Easy - A Handy Debugging Module With Colorized Output and Formatting
  $debug->debug('INFO',     ['Information and VERBOSE mode message']);
  $debug->debug('DEBUG',    ['Level 1 Debug message']);
  $debug->debug('DEBUGMAX', ['Level 2 Debug message']);
- 
+
+ # Or...
+
+ $debug->ERR(        ['Error message']);
+ $debug->ERROR(      ['Error message']);
+ $debug->WARN(       ['Warning message']);
+ $debug->WARNING(    ['Warning message']);
+ $debug->NOTICE(     ['Notice message']);
+ $debug->INFO(       ['Information and VERBOSE mode message']);
+ $debug->INFORMATION(['Information and VERBOSE mode message']);
+ $debug->DEBUG(      ['Level 1 Debug message']);
+ $debug->DEBUGMAX(   ['Level 2 Debug message']);
+
  my @messages = (
     'First Message',
     'Second Message',
@@ -161,7 +180,9 @@ Data::Dumper
 
 END {    # We spit out one last message before we die, the total execute time.
     my $bench = colored(['bright_cyan on_black'], sprintf('%06s', sprintf('%.02f', (time - $MASTERSTART))));
-    $LOG->DEBUG(' %s%s %s', $bench, $ANSILevel{'DEBUG'}, colored(['black on_white'], "---- $0 complete ----")) if (defined($ANSILevel{'DEBUG'}) && defined($LOG));
+    my $name  = $SCRIPTNAME;
+    $name .= ' [child]' if ($PARENT ne $$);
+    $LOG->DEBUG(' %s%s %s', $bench, $ANSILevel{'DEBUG'}, colored(['black on_white'], "---- $name complete ----")) if (defined($ANSILevel{'DEBUG'}) && defined($LOG));
 }
 
 =head1 METHODS
@@ -250,15 +271,21 @@ output.
 
 A string that is parsed into the output prefix.
 
-DEFAULT:  %D %T %B %L %P
+DEFAULT:  '%D %T %B %L[%P][%l] '
 
-%D = Date (Uses format of "DateStamp" below
-%T = Time (Uses format of "TimeStamp" below
+%D = Date (Uses format of "DateStamp" below)
+%T = Time (Uses format of "TimeStamp" below)
 %B = Benchmark
 %L = Log Level
+%l = Line Number
 %P = Module and subroutine of call
+%p = Process ID
 %d = Just Date (typically used internally only, use %D)
 %t = Just time (typically used internally only, use %T)
+%F = Script Filename (parsed $0)
+%f = Running in parent or child?
+     P = Parent
+     C = Child
 
 =item B<TimeStamp> [pattern]
 
@@ -372,7 +399,8 @@ sub new {
         'Color'              => 1,                                    # Default to colorized output
         'DateStamp'          => colored(['yellow on_black'], '%d'),
         'TimeStamp'          => colored(['yellow on_black'], '%t'),
-        'Padding'            => -25,                                  # Default padding is 25 spaces
+        'Padding'            => -15,                                  # Default padding is 15 spaces
+        'CPadding'           => -2,
         'Prefix'             => undef,
         'Filename'           => '[' . colored(['magenta'],$filename) . ']',
         'ANSILevel'          => {
@@ -422,7 +450,7 @@ sub new {
 
     # This assembles the Time and Date stamping output.  If any of the
     # variables are blank, then that part will be disabled.
-    $self->{'PREFIX'} = '%D %T %B %L[%P] ' unless (defined($self->{'PREFIX'}));
+    $self->{'PREFIX'} = '%D %T %B %L[%P][%l] ' unless (defined($self->{'PREFIX'}));
     # The Global loglevel is set here.
     if ($self->{'LOGLEVEL'} eq 'VERBOSE') {
         $self->{'LOG'}->level('INFO');
@@ -441,7 +469,9 @@ sub new {
         }
     );
     # Signal the script has started (and logger initialized)
-    $self->{'LOG'}->DEBUG('   %.02f%s %s', 0, $self->{'ANSILEVEL'}->{'DEBUG'}, colored(['black on_white'], "----- $0 begin -----"));
+    my $name  = $SCRIPTNAME;
+    $name .= ' [child]' if ($PARENT ne $$);
+    $self->{'LOG'}->DEBUG('   %.02f%s %s', 0, $self->{'ANSILEVEL'}->{'DEBUG'}, colored(['black on_white'], "----- $name begin -----"));
     bless($self, $class);
     return ($self);
 } ## end sub new
@@ -469,63 +499,112 @@ sub debug {
     my $level = uc(shift);
     my $msgs  = shift;
 
-    if ($level !~ /ERR.*|WARN|NOTICE|INFO|DEBUG/i) { # Compatibility with older versions.
+    if ($level !~ /ERR.*|WARN.*|NOTICE|INFO.*|DEBUG/i) { # Compatibility with older versions.
         $level = $msgs;
-        $msgs = shift;
+        $msgs  = shift;
     }
-    my @messages;
+    $level = 'ERR'  if ($level eq 'ERROR');      # This allows the programmer to use either
+    $level = 'WARN' if ($level eq 'WARNING');
+    $level = 'INFO' if ($level eq 'INFORMATION');
 
+    # A much quicker bypass when the log level is below what is needed
+    return if ($LevelLogic{$self->{'LOGLEVEL'}} < $LevelLogic{$level});
+
+    my @messages;
     if (ref($msgs) eq 'SCALAR' || ref($msgs) eq '') {
         push(@messages, $msgs);
     } elsif (ref($msgs) eq 'ARRAY') {
         @messages = @{$msgs};
     } else {
         if ($Modules->{'Data::Printer'}) {
-            push(@messages, p($msgs));
+            eval {
+                push(@messages, p($msgs));
+            };
         } else {
-            push(@messages, Dumper($msgs));
+            eval {
+                push(@messages, Dumper($msgs));
+            };
         }
     }
-    my ($p,$f,$cline) = caller; # We get the real line number from here (hopefully).
-    my ($package, $filename, $line, $subroutine, $hasargs, $wantarray, $evaltext, $is_require, $hints, $bitmask) = caller(1);
-    if (!defined($subroutine) || $subroutine eq '') {    # If there is no subroutine name, then it's usually the "main" namespace
-        $subroutine = 'main';
+    my ($sname,$cline,$nested,$subroutine,$thisBench,$thisBench2) = ('','','','','','');
+    # Figure out the proper caller tree and line number ladder
+    # But only if it's part of the prefix, else don't waste time.
+    if ($self->{'PREFIX'} =~ /\%(P|l)/) { # %P = Subroutine, %l = Line number(s)
+        my $package = '';
+        my $count   = 1;
+        my $nest    = 0;
+        while (my @array = caller($count)) {
+            if ($array[3] !~ /Debug::Easy/) {
+                $package       = $array[0];
+                my $subroutine = $array[3];
+                $subroutine =~ s/^$package\:\://;
+                $sname      =~ s/$subroutine//;
+                if ($sname eq '') {
+                    $sname = ($subroutine ne '') ? $subroutine : $package;
+                    $cline = $array[2];
+                } else {
+                    $sname = $subroutine . '::' . $sname;
+                    $cline = $array[2] . '/' . $cline;
+                }
+                $nest++;
+            }
+            $count++;
+        }
+        if ($package ne '') {
+            $sname = $package . '::' . $sname;
+            $nested = ' ' x $nest if ($nest);
+        } else {
+            my @array = caller(1);
+            $cline = $array[2];
+            if ($cline eq '') {
+                @array = caller(0);
+                $cline = $array[2];
+            }
+            $sname = 'main';
+        }
+        $subroutine = $sname;
+        $self->{'PADDING'}  = 0 - length($subroutine) if (length($subroutine) > abs($self->{'PADDING'}));
+        $self->{'CPADDING'} = 0 - length($cline) if (length($cline) > abs($self->{'CPADDING'}));
+        $cline              = sprintf('%' . $self->{'CPADDING'} . 's',$cline);
+        $subroutine         = colored(['bold cyan'],sprintf('%' . $self->{'PADDING'} . 's',$subroutine));
     }
-    $cline         = sprintf('%04d',$cline);
-    my $thisBench  = sprintf('%7s', sprintf(' %.02f', time - $self->{$level . '_LASTSTAMP'}));
-    my $thisBench2 = ' ' x length($thisBench);
-
-    # For multiline output, only output the bench data on the first line.  Use padded spaces for the rest.
-    my $nested  = do {my $n = 0; 1 while caller(2 + $n++); ' ' x $n};    # add padding to nested lines
-    $subroutine = $nested . $subroutine;
-    $self->{'PADDING'} = 0 - length($subroutine) if (length($subroutine) >= abs($self->{'PADDING'}));
-    $subroutine = colored(['bold cyan'],sprintf('%' . $self->{'PADDING'} . 's',$subroutine));
-    my $first = 1;                                                      # Set the first line flag.
-    foreach my $msg (@messages) {                                       # Loop through each line of output and format accordingly.
+    # Figure out the benchmarks, but only if it is in the prefix
+    if ($self->{'PREFIX'} =~ /\%B/) {
+        # For multiline output, only output the bench data on the first line.  Use padded spaces for the rest.
+        $thisBench  = sprintf('%7s', sprintf(' %.02f', time - $self->{$level . '_LASTSTAMP'}));
+        $thisBench2 = ' ' x length($thisBench);
+    }
+    my $first = 1;                         # Set the first line flag.
+    foreach my $msg (@messages) {          # Loop through each line of output and format accordingly.
         if (ref($msg) ne '') {
             if ($Modules->{'Data::Printer'}) {
-                $msg = p($msg);
+                eval {
+                    $msg = p($msg);
+                };
             } else {
-                $msg = Dumper($msg);
+                eval {
+                    $msg = Dumper($msg);
+                };
             }
         }
-        if ($msg =~ /\n/s) {                                            # If the line contains newlines, then it too must be split into multiple lines.
+        if ($msg =~ /\n/s) {               # If the line contains newlines, then it too must be split into multiple lines.
             my @message = split(/\n/, $msg);
-            foreach my $line (@message) {                               # Loop through the split lines and format accordingly.
-                $self->_send_to_logger($level, $line, $first, $thisBench, $thisBench2, $subroutine, $cline);
-                $first = 0;                                             # Clear the first line flag.
+            foreach my $line (@message) {  # Loop through the split lines and format accordingly.
+                $self->_send_to_logger($level, $nested, $line, $first, $thisBench, $thisBench2, $subroutine, $cline);
+                $first = 0;                # Clear the first line flag.
             }
-        } else {    # This line does not contain newlines.  Treat it as a single line.
-            $self->_send_to_logger($level, $msg, $first, $thisBench, $thisBench2, $subroutine, $cline);
+        } else {                           # This line does not contain newlines.  Treat it as a single line.
+            $self->_send_to_logger($level, $nested, $msg, $first, $thisBench, $thisBench2, $subroutine, $cline);
         }
-        $first = 0;    # Clear the first line flag.
+        $first = 0;                        # Clear the first line flag.
     }
     $self->{$level . '_LASTSTAMP'} = time;
-} ## end sub debug
+}
 
-sub _send_to_logger {    # This actually simplifies the previous class
+sub _send_to_logger {    # This actually simplifies the previous class ... seriously
     my $self       = shift;
     my $level      = shift;
+    my $padding    = shift;
     my $msg        = shift;
     my $first      = shift;
     my $thisBench  = shift;
@@ -533,39 +612,41 @@ sub _send_to_logger {    # This actually simplifies the previous class
     my $subroutine = shift;
     my $cline      = shift;
 
-    $level = 'ERR' if ($level eq 'ERROR'); # This allows the programmer to use either
-    my $dt = DateTime->now();
-    my $Date = $dt->ymd();
-    my $Time = $dt->hms();
+    my $dt     = DateTime->now();
+    my $Date   = $dt->ymd();
+    my $Time   = $dt->hms();
     my $prefix = $self->{'PREFIX'} . ''; # A copy not a pointer
+    my $forked = ($PARENT ne $$) ? 'C' : 'P';
 
+    $prefix =~ s/\%p/$$/g;
     $prefix =~ s/\%L/$ANSILevel{$level}/g;
+    $prefix =~ s/\%l/$cline/g;
     $prefix =~ s/\%D/$self->{'DATESTAMP'}/g;
     $prefix =~ s/\%T/$self->{'TIMESTAMP'}/g;
     $prefix =~ s/\%d/$Date/g;
     $prefix =~ s/\%t/$Time/g;
     $prefix =~ s/\%F/$self->{'FILENAME'}/g;
+    $prefix =~ s/\%f/$forked/g;
+    $prefix =~ s/\%P/$subroutine/g;
     if ($first) {
-        $prefix =~ s/\%P/$subroutine/g;
         $prefix =~ s/\%B/$thisBench/g;
     } else {
-        $prefix =~ s/\%P/$subroutine /g;
         $prefix =~ s/\%B/$thisBench2/g;
     }
 
-    if ($level eq 'INFO' && $self->{'LOGLEVEL'} eq 'VERBOSE') {    # Trap verbose flag and temporarily drop the prefix.
+    if ($level eq 'INFO' && $self->{'LOGLEVEL'} eq 'VERBOSE') { # Trap verbose flag and temporarily drop the prefix.
         $self->{'LOG'}->INFO($msg);
-    } elsif ($level eq 'DEBUGMAX') {                               # Special version of DEBUG.  Outputs as DEBUG in Log::Fast
+    } elsif ($level eq 'DEBUGMAX') {                            # Special version of DEBUG.  Outputs as DEBUG in Log::Fast
         if ($self->{'LOGLEVEL'} eq 'DEBUGMAX') {
-            $self->{'LOG'}->DEBUG($prefix . $msg);
+            $self->{'LOG'}->DEBUG($prefix . $padding . $msg);
         }
-    } elsif ($level eq 'DEBUGWAIT') {                              # Same as above, but waits for an [ENTER] keypress.  HIGHLY EXPERIMENTAL, no whining
+    } elsif ($level eq 'DEBUGWAIT') {                           # Same as above, but waits for an [ENTER] keypress.  HIGHLY EXPERIMENTAL, no whining
         if ($self->{'LOGLEVEL'} eq 'DEBUGWAIT') {
-            $self->{'LOG'}->DEBUG($prefix . $msg);
-            <>; # Wait for [ENTER] keypress
+            $self->{'LOG'}->DEBUG($prefix . $padding . $msg);
+            <>; # Wait for [ENTER] keypress.  Seriously, it's experimental, I said no whining.
         }
     } else {
-        $self->{'LOG'}->$level($prefix . $msg);
+        $self->{'LOG'}->$level($prefix . $padding . $msg);
     }
 }
 
@@ -733,7 +814,7 @@ sub DEBUGMAX {
 
 =head2 DEBUGWAIT
 
-EXPERIMENTAL!!!!  No whining.
+EXPERIMENTAL!!!!  I don't even know why I included it in the docs.  So no whining.
 
 This is a friendlier wrapper to 'debug('DEBUGWAIT',['message'])'.  The level is inferred by its name.
 
@@ -751,6 +832,12 @@ sub DEBUGWAIT {
     $self->debug('DEBUGWAIT',@_);
 }
 
+=head1 CAVEATS
+
+Since it is possible to duplicate the object in a fork or thread, the output
+formatting may be mismatched between forks and threads due to the automatic
+padding adjustment of the subroutine name field.
+
 =head1 INSTALLATION
 
 To install this module, run the following commands:
@@ -778,7 +865,7 @@ modify it under the same terms as Perl itself.
 
 =head1 VERSION
 
-Version 0.34    (June 01, 2015)
+Version 0.36    (June 13, 2015)
 
 =head1 BUGS
 
